// Original code is from Chad Phillips: http://www.windmeadow.com/node/42
// Modified by Fabien Goy - fabien.goy at free . fr
// August, 29 2007

#include <Wire.h>
//#include <string.h>
//#include <stdio.h>

#define USE_BYTES

uint8_t outbuf[6]; // array to store arduino output
int cnt = 0;
int ledPin = 13;

void
setup ()
{
beginSerial (9600*2);
Serial.print ("Finished setup\n");
Wire.begin (); // join i2c bus with address 0x52
nunchuck_init (); // send the initilization handshake

while (Wire.available ())
{
outbuf[cnt] = nunchuk_decode_byte (Wire.receive ()); // receive byte as an integer
cnt++;
}
cnt = 0;
}

void
nunchuck_init ()
{
Wire.beginTransmission (0x52); // transmit to device 0x52
Wire.send (0x40); // sends memory address
Wire.send (0x00); // sends sent a zero.
Wire.endTransmission (); // stop transmitting
}

void
send_zero ()
{
digitalWrite (ledPin, LOW);
Wire.beginTransmission (0x52); // transmit to device 0x52
Wire.send (0x00); // sends one byte
Wire.endTransmission (); // stop transmitting
}

void
loop ()
{
Wire.requestFrom (0x52, 6); // request data from nunchuck
while (Wire.available ())
{
outbuf[cnt] = nunchuk_decode_byte (Wire.receive ()); // receive byte as an integer
digitalWrite (ledPin, HIGH); // sets the LED on
cnt++;
}

// If we recieved the 6 bytes, then go print them
if (cnt >= 5)
{
print ();
}

cnt = 0;
send_zero (); // send the request for next bytes
delay (25);
}

// Print the input data we have recieved
// accel data is 10 bits long
// We only send the 8 most significant bits
// If need the 10 bits int, you could write this:
// int accel_x_axis = outbuf[2] <<4;
// if ((outbuf[5] >> 2) & 1)
// accel_x_axis += 2;
// if ((outbuf[5] >> 3) & 1)
// accel_x_axis += 1;
// but be careful that the serial only send
// the 8 least significant bits of an int

void
print ()
{
int joy_x_axis = outbuf[0];
int joy_y_axis = outbuf[1];
int accel_x_axis = outbuf[2];
int accel_y_axis = outbuf[3];
int accel_z_axis = outbuf[4];

int z_button = 0;
int c_button = 0;

// byte outbuf[5] contains bits for z and c buttons
if ((outbuf[5] >> 0) & 1)
{
z_button = 1;
}
if ((outbuf[5] >> 1) & 1)
{
c_button = 1;
}

#ifdef USE_BYTES
// We send the data over serial in BYTE mode :
// can't be displayed directly on a terminal
Serial.print (joy_x_axis == 0xFF ? 0xFE : joy_x_axis, BYTE);
Serial.print (joy_y_axis == 0xFF ? 0xFE : joy_y_axis, BYTE);
Serial.print (accel_x_axis == 0xFF ? 0xFE : accel_x_axis, BYTE);
Serial.print (accel_y_axis == 0xFF ? 0xFE : accel_y_axis, BYTE);
Serial.print (accel_z_axis == 0xFF ? 0xFE : accel_z_axis, BYTE);
Serial.print (z_button, BYTE);
Serial.print (c_button, BYTE);
Serial.print (0xFF,BYTE);
delay(10);
#else
Serial.print (joy_x_axis);
Serial.print (' ');
Serial.print (joy_y_axis);
Serial.print (' ');
Serial.print (accel_x_axis);
Serial.print (' ');
Serial.print (accel_y_axis);
Serial.print (' ');
Serial.print (accel_z_axis);
Serial.print (' ');
Serial.print (z_button);
Serial.print (' ');
Serial.println (c_button);
delay(100);
#endif
}

// Encode data to format that most wiimote drivers except
// only needed if you use one of the regular wiimote drivers
char
nunchuk_decode_byte (char x)
{
x = (x ^ 0x17) + 0x17;
return x;
}
